/* --COPYRIGHT--,BSD
 * Copyright (c) 2016, Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * --/COPYRIGHT--*/

/********************************************************************************
 *
 *  DRV8889.c - Definition file for utility functions
 *  DRV8889-Q1_Example FIRMWARE
 *  11/29/2018
 *
 *******************************************************************************/

/********************************************************************************
 * HEADER FILES
 *******************************************************************************/
#include <drv8889.h>
#include <SPI.h>
#include <stepper.h>
/********************************************************************************
 * LOCAL ROUTINES
 *******************************************************************************/
/*
 * SPI Initialization
 */
void SPI_Init(void)
{
    P3OUT |= nSCS;
    P3DIR |= nSCS;

    P3SEL |= (SCLK | SDO | SDI);

    UCB0CTL1 = UCSWRST;
    // UCB0CTL0 |= UCCKPH + UCMSB + UCMST + UCSYNC; // 3-pin, 8-bit SPI master
    UCB0CTL0 |= UCMSB | UCMST | UCSYNC;
    UCB0CTL1 |= UCSSEL_2; // SMCLK
    UCB0BR0 |= 0x14; // /100KHz
    UCB0BR1 = 0; //
    UCB0CTL1 &= ~UCSWRST; // **Initialize USCI state machine**

    // 5) Pulse nSCS to clear shift DRV8873X-Q1 register
     P3OUT &= ~nSCS;    //SPI Set
     _delay_cycles(10);
     P3OUT |= nSCS;     //SPI Reset
}

/*
 * This is SPI interface, used to read the device configurations,parameters and
 * status information for S version of device.
 * Register format |R/W|A4|A3|A2|A1|A0|*|*|D7|D6|D5|D4|D3|D2|D1|D0|
 * Ax is address bit, Dx is data bits and R/W is read write bit.
 * For read R/W bit should be 1.
 */
uint16_t SPI_readRegister(uint8_t address)
{
  // disable UART interrupt
//  IE2 &= ~UCA0RXIE;
  volatile uint16_t reg_value = 0;
  volatile uint8_t dataMSB    = 0;
  volatile uint8_t dataLSB    = 0;

  reg_value |= SPI_RW_BIT_MASK;                                      /*Set R/W bit*/
  reg_value |= ((address << SPI_ADDRESS_POS) & SPI_ADDRESS_MASK);    /* Configure register address value*/

  set_nSCS_lo;//SPI set

  while (!(IFG2 & UCB0TXIFG));                                       /* USCI_B0 TX buffer ready*/
  UCB0TXBUF = (uint8_t)((reg_value >>8) & 0xFF);                        /* Transmit the Address(MSB Byte)*/

  while(UCB0STAT & SPI_BUSY_FLAG)
  {
    ;                                                                /* Wait till Transmission is complete*/
  }
  dataMSB = UCB0RXBUF;                                               /* Recieve the First byte of data, MSB byte*/

  UCB0TXBUF = 0x00;                                                  /* Transmit Second byte*/
  while(UCB0STAT & SPI_BUSY_FLAG)
  {
    ;                                                                /* Wait till Transmission is complete*/
  }
  dataLSB = UCB0RXBUF & 0xFF;                                        /* Recieve the Second byte,LSB byte*/

  set_nSCS_hi; //SPI Reset
  _delay_cycles(10);

  reg_value = ((((dataMSB<<8) | dataLSB) & SPI_DATA_MASK)>>SPI_DATA_POS); /* complete data */

  // Enable UART interrupt
//  IE2 |= UCA0RXIE;

  return (reg_value);
}

/*
 * This is SPI interface, used to write the set device configurations and operating
 * parameters of S version of device.
 * Register format |R/W|A4|A3|A2|A1|A0|*|*|D7|D6|D5|D4|D3|D2|D1|D0|
 * Ax is address bit, Dx is data bits and R/W is read write bit.
 * For write R/W bit should 0.
 */
uint16_t SPI_writeRegister(uint8_t address, uint16_t data)
{
  // disable UART interrupt
//  IE2 &= ~UCA0RXIE;
  volatile uint16_t reg_value = 0;

//  reg_value &=~ SPI_RW_BIT_MASK;                                        /*Clear R/W bit*/
  reg_value |= ((address << SPI_ADDRESS_POS) & SPI_ADDRESS_MASK);         /* Adding register address value*/
  reg_value |= ((data << SPI_DATA_POS) & SPI_DATA_MASK);                  /*Adding data value */

  set_nSCS_lo;                                                         /*SPI set*/

  while (!(IFG2 & UCB0TXIFG));                                            /* USCI_B0 TX buffer ready?*/

  UCB0TXBUF = (uint8_t)((reg_value>>8) & 0xFF);                           /* Transmit first Byte,MSB-Byte*/

  while (!(IFG2 & UCB0TXIFG));                                            /* USCI_B0 TX buffer ready*/

  UCB0TXBUF = (uint8_t)(reg_value & 0xFF);                                /* Transmit Second Byte, LSB-Byte*/

  while(UCB0STAT & SPI_BUSY_FLAG)
  {
    ;                                                                     /* Wait till Transmission is complete*/
  }

  set_nSCS_hi;                                                          /*SPI Reset*/
  _delay_cycles(10);

  if(address == SPI_REG_CTRL3) DetermineMicrostepping(data);

  // Enable UART interrupt
//  IE2 |= UCA0RXIE;

  return 0;
}
