/* --COPYRIGHT--,BSD
 * Copyright (c) 2016, Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * --/COPYRIGHT--*/

/*******************************************************************************
 *
 *  drv8889.h - Declaration file for utility functions and global variables
 *  DRV8889_EVM_FIRMWARE
 *  1/2/2019
 *
 ******************************************************************************/
#include "../../dataTypeDefinition.h"


/*******************************************************************************
 * MACROs
 *******************************************************************************/

/*************************************************************
* DRV8889-Q1 SPI Register ADDRESS
*************************************************************/

#define SPI_REG_FAULT_STATUS         (uint8_t)(0x00)         /* Fault Status Register */
#define SPI_REG_DIAG_STATUS_1        (uint8_t)(0x01)         /* Diagnostic register 1 for OCP   */
#define SPI_REG_DIAG_STATUS_2        (uint8_t)(0x02)         /* Diagnostic register 2 for other faults and warnings   */
#define SPI_REG_CTRL1                (uint8_t)(0x03)         /* CTRL1 register - Torque DAC, synchronous rectification, slew rate  */
#define SPI_REG_CTRL2                (uint8_t)(0x04)         /* CTRL2 register - Disable, TOFF, DECAY  */
#define SPI_REG_CTRL3                (uint8_t)(0x05)         /* CTRL3 register - Step, dir, microstep  */
#define SPI_REG_CTRL4                (uint8_t)(0x06)         /* CTRL4 register - Clear fault, lock, OL, OCP, OTSD, OTW  */
#define SPI_REG_CTRL5                (uint8_t)(0x07)         /* CTRL5 register - Stall detection configuration  */
#define SPI_REG_CTRL6                (uint8_t)(0x08)         /* CTRL6 register - Stall threshold */
#define SPI_REG_CTRL7                (uint8_t)(0x09)         /* CTRL7 register - Torque count */
#define SPI_REG_CTRL8                (uint8_t)(0x0A)         /* CTRL8 register - UTW, Rev ID */

/* Mask for Register Bit Definitions */

/* SPI_REG_00 : IC fault Status Register */
#define OL_MASK               (0x01)         /*Indicates open load condition*/
#define OT_MASK               (0x02)         /*Logic OR of OTW, UTW, and OTSD.*/
#define STL_SLIP_MASK         (0x04)         /*Indicates motor stall or slip*/
#define OCP_MASK              (0x08)         /*Indicates undervoltage lockout fault condition*/
#define CPUV_MASK             (0x10)         /*Indicates charge pump undervoltage fault condition*/
#define UVLO_MASK             (0x20)         /*Indicates an undervoltage condition*/
#define SPI_ERROR_MASK        (0x40)         /*Indication SPI communication error*/
#define FAULT_MASK            (0x80)         /*Inverse of the nFAULT pin*/


/* SPI_REG_01 : Diagnostic register 1 for OCP */
#define OCP_HS1_A_MASK        (0x01)        /*Indicates overcurrent fault on the high-side FET of AOUT1 */
#define OCP_LS1_A_MASK        (0x02)        /*Indicates overcurrent fault on the low-side FET of AOUT1 */
#define OCP_HS2_A_MASK        (0x04)        /*Indicates overcurrent fault on the high-side FET of AOUT2 */
#define OCP_LS2_A_MASK        (0x08)        /*Indicates overcurrent fault on the low-side FET of AOUT2 */
#define OCP_HS1_B_MASK        (0x10)        /*Indicates overcurrent fault on the high-side FET of BOUT1 */
#define OCP_LS1_B_MASK        (0x20)        /*Indicates overcurrent fault on the low-side FET of BOUT1 */
#define OCP_HS2_B_MASK        (0x40)        /*Indicates overcurrent fault on the high-side FET of BOUT2 */
#define OCP_LS2_B_MASK        (0x80)        /*Indicates overcurrent fault on the low-side FET of BOUT2 */

/* SPI_REG_02 : Diagnostic register 2 for other faults and warnings */
#define OL_A_MASK             (0x01)        /*Indicates open-load detection on AOUT*/
#define OL_B_MASK             (0x02)        /*Indicates open-load detection on BOUT*/
#define SLIP_MASK             (0x04)        /*Indicates motor slip condition*/
#define STALL_MASK            (0x08)        /*Indicates motor stall condition*/
#define STL_LRN_OK_MASK       (0x10)        /*Indicates stall detection learning is successful*/
#define OTS_MASK              (0x20)        /*Indicates overtemperature shutdown*/
#define OTW_MASK              (0x40)        /*Indicates overtemperature warning */
#define UTW_MASK              (0x80)        /*Indicates undertemperature warning*/

/* SPI_REG_03 : CTRL1 register - Torque DAC, synchronous rectification, slew rate */
#define SLEW_RATE_MASK        (0x03)        /*Slew rate selection bits*/
#define EN_NSR_MASK           (0x04)        /*Enable synchronous rectification bits*/
#define RSVD1_MASK            (0x08)        /*TI reserved bit*/
#define TRQ_DAC_MASK          (0xF0)        /*Torque DAC bits*/

/* SPI_REG_04 : CTRL2 register - Disable, TOFF, DECAY */
#define DECAY_MASK            (0x07)        /*Bridge decay setting */
#define TOFF_MASK             (0x18)        /*Current regulation TOFF setting*/
#define RSVD2_MASK            (0x60)        /*TI reserved bits*/
#define DIS_OUT_MASK          (0x80)        /*Hi-Z outputs bit. OR-ed with DRVOFF*/

/* SPI_REG_05 : CTRL3 register - Step, dir, microstep */
#define MICROSTEP_MODE_MASK   (0x0F)        /*Microstep setting*/
#define SPI_STEP_MASK         (0x10)        /*Enable SPI step control mode*/
#define SPI_DIR_MASK          (0x20)        /*Enable SPI direction control mode*/
#define STEP_MASK             (0x40)        /*Step control bit if SPI_STEP is enabled*/
#define DIR_MASK              (0x80)        /*Direction control bit if SPI_DIR is enabled*/

/* SPI_REG_06 : CTRL4 register - Clear fault, lock, OL, OCP, OTSD, OTW */
#define TW_REP_MASK           (0x01)        /*Report OTW and UTW on nFAULT*/
#define OTSD_MODE_MASK        (0x02)        /*OTSD latch fault setting*/
#define OCP_MODE_MASK         (0x04)        /*OCP latch fault setting*/
#define EN_OL_MASK            (0x08)        /*Open load detection enable*/
#define LOCK_MASK             (0x70)        /*Lock SPI registers*/
#define CLR_FLT_MASK          (0x80)        /*Clear all fault bits*/

/* SPI_REG_07 : CTRL5 register - Stall detection configuration */
#define SLIP_REP_MASK         (0x01)        /*Report slip detection on nFAULT*/
#define EN_SLIP_MASK          (0x02)        /*Enable slip detection*/
#define EN_STLFCHG_MASK       (0x04)        /*Enable stall detection during frequency change*/
#define STL_REP_MASK          (0x08)        /*Report stall detection on nFAULT*/
#define DIS_STL_MASK          (0x10)        /*Enable stall detection*/
#define STL_LRN_MASK          (0x20)        /*Learn stall count threshold*/
#define STL_MODE_MASK         (0x40)        /*Stall mode*/
#define STL_UPDATE_MASK       (0x80)        /*TBD*/

/* SPI_REG_08 : CTRL6 register - Stall threshold */
#define STALL_TH_MASK         (0xFF)        /*Stall Threshold*/

/* SPI_REG_09 : CTRL7 register - Torque count */
#define TRQ_COUNT_MASK        (0xFF)        /*Torque Count*/

/* SPI_REG_0A : CTRL8 register - UTW, Rev ID */
#define REV_ID_MASK           (0x0F)        /*Revision ID*/
#define STL_UPDATE_MODE_MASK  (0x10)        /*TBD*/
#define UTW_REP_MASK          (0x20)        /*Report UTW on nFAULT*/
#define RSVD3_MASK            (0xC0)        /*TI reserved*/

/*******************************************************************************
 * Structure
 *******************************************************************************/

/**
 *  Structure initializing the DRV8889-Q1 Register.
 */
typedef struct DRV8889_REG
{
  /* Registers mapped to DRV8889-Q1 SPI registers*/
  uint8_t fault_status_reg;       /* Fault Status Register, Address = 0x00h*/
  uint8_t diag_status1_reg;       /* Diagnostic register 1 for OCP, Address = 0x01h*/
  uint8_t diag_status2_reg;       /* Diagnostic register 2 for other faults and warnings, Address = 0x02h*/
  uint8_t ctrl1_reg;              /* CTRL1 register - Torque DAC, synchronous rectification, slew rate, Address = 0x03h*/
  uint8_t ctrl2_reg;              /* CTRL2 register - Disable, TOFF, DECAY, Address = 0x04h*/
  uint8_t ctrl3_reg;              /* CTRL3 register - Step, dir, microstep, Address = 0x05h*/
  uint8_t ctrl4_reg;              /* CTRL4 register - Clear fault, lock, OL, OCP, OTSD, OTW, Address = 0x06h*/
  uint8_t ctrl5_reg;              /* CTRL5 register - Stall detection configuration, Address = 0x03h*/
  uint8_t ctrl6_reg;              /* CTRL6 register - Stall threshold, Address = 0x04h*/
  uint8_t ctrl7_reg;              /* CTRL7 register - Torque count, Address = 0x05h*/
  uint8_t ctrl8_reg;              /* CTRL8 register - UTW, Rev ID, Address = 0x06h*/

} DRV8889_Q1_REG_t;
