/* --COPYRIGHT--,BSD
 * Copyright (c) 2016, Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * --/COPYRIGHT--*/

/*******************************************************************************
 *
 *  DRV8889.h - Declaration file for utility functions and global variables
 *  DRV8889-Q1EVM_FIRMWARE
 *  1/2/2019
 *
 ******************************************************************************/
 #ifndef drv8889q1_H_
 #define drv8889q1_H_

/******************************************************************************
 * HEADER FILES
 *****************************************************************************/
#include "drv8889-RegMap.h"
#include "msp430.h"

/******************************************************************************
 * MACROS
 *****************************************************************************/

/*
 * GPIO Port 1 Definitions
 */
#define nSLEEP             BIT0         /**<P1.0: Sleep pin*/
#define DRVOFF             BIT1         /**<P1.1: DRVOFF enable pin*/
#define DIR                BIT2         /**<P1.2: DIRection pin*/
#define STEP               BIT6         /**<P1.6: STEP pin*/

/*
 * GPIO Port 2 Definitions
 */
#define nFAULT_INT         BIT7         // P2.7: Fault indication pin (with port interrupt)

/*
 * GPIO Port 3 Definitions
 */
#define nSCS               BIT0         // P3.0
#define SDI                BIT1         // P3.1
#define SDO                BIT2         // P3.2
#define SCLK               BIT3         // P3.3
#define RX                 BIT4         // P3.4
#define TX                 BIT5         // P3.5

/*
 * GPIO Port 4 Definitions
 */
#define nFAULT             BIT6         // P4.6: Fault indication pin

/*
 * GPIO Port 6 Definitions
 */
#define STATUS             BIT5         // P6.5 STATUS LED
#define VREF               BIT7         // P6.7 DAC for VREF

/*Firmware version*/
#define FW_VERSION   1.0

/* Enable nSLEEP*/
#define set_nSLEEP_hi            P1OUT |= nSLEEP
#define set_nSLEEP_lo            P1OUT &= ~nSLEEP

/* Enable DRVOFF*/
#define set_DRVOFF_hi            P1OUT |= DRVOFF
#define set_DRVOFF_lo            P1OUT &= ~DRVOFF

/* Enable DIR*/
#define set_DIR_hi               P1OUT |= DIR
#define set_DIR_lo               P1OUT &= ~DIR

/* Enable STEP*/
#define set_STEP_hi               P1OUT |= STEP
#define set_STEP_lo               P1OUT &= ~STEP

#define read_nFAULT_pin           (P4IN & nFAULT) + (P2IN & nFAULT_INT)
#define read_nFAULT_INT_pin       (P2IN & nFAULT_INT)
#define toggle_status_led         P6OUT ^=  STATUS

/******************************************************************************
* DATA STRUCTURES
 *****************************************************************************/

typedef struct DRV8889_Device_Obj
{

  uint16_t DeviceID; // DRV8889-Q1 - Read IC_ID register and set the value
  float Firmware_Version;
  /* Device Parameters*/
  volatile uint8_t nSleep;     /**< 0: Device Sleep , 1: Device Awake */
  volatile uint8_t gDRVOFF;    /**< 0: Outputs active, 1: Outputs Hi-Z */
  volatile uint8_t nFault;     /**< 0: No fault , 1: Fault */
  volatile uint8_t ClrFlt;

  /* Register Read Write */
   uint16_t   ManWriteAddr;
   uint16_t   ManReadAddr;
   uint16_t   ManWriteData;
   uint16_t   ManReadData;
   bool       ManWriteCmd;
   bool       ManReadCmd;
   bool       ReadCmd;

} DRV8889_Device_Obj_t;

extern volatile DRV8889_Device_Obj_t gDRV8889_Obj;


/******************************************************************************
 * EXTERN DECLARATIONS
 *****************************************************************************/
//extern volatile TRGT_INFO_Obj targetInfo;
extern volatile DRV8889_Q1_REG_t gDeviceSpiReg;
void UpdateGUI_Register();
/*****************************************************************************/

#ifdef __cplusplus
extern "C" {
#endif

/******************************************************************************
 * API Prototypes
 *****************************************************************************/
void mcu_init();
/**
* @brief     :This function, initializes all the peripherals and DRV8889-Q1 related GPIOs and registers\n
 * @param[in] void There is no argument
 * @return    void (none)
 */
void drv8889q1_init(void);
/**
 * @brief     :Read and return the DRV89xxS SPI register value\n
 * @param[in] addr, Address of SPI resister
 * @return    uint16_t, SPI register value
 */

/*****************************************************************************/

/* Support for C++ codebase */
#ifdef __cplusplus
}
#endif

#endif /* drv89xxq1_H_ */
