/* --COPYRIGHT--,BSD
 * Copyright (c) 2016, Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * --/COPYRIGHT--*/

/********************************************************************************
 *
 *  DRV8889.c - Definition file for utility functions
 *  DRV8889-Q1EVM_FIRMWARE
 *  1/2/2019
 *
 *******************************************************************************/

/********************************************************************************
 * HEADER FILES
 *******************************************************************************/
#include <drv8889.h>
#include <SPI.h>
#include <stepper.h>
#include "uart.h"
/********************************************************************************
 * MACROS
 *******************************************************************************/
#pragma DATA_SECTION(gDRV8889_Obj,".drvVarSection");
#pragma DATA_SECTION(gDeviceSpiReg,".drvRegSection");
volatile DRV8889_Q1_REG_t gDeviceSpiReg;
volatile DRV8889_Device_Obj_t gDRV8889_Obj;

/********************************************************************************
 * DATA
 *******************************************************************************/

/********************************************************************************
 * LOCAL ROUTINES
 *******************************************************************************/

/*
 * Clock Initialization
 */
static inline void clock_Init(void)
{
     DCOCTL  = CALDCO_8MHZ;          // Set DCO to 8MHz

     BCSCTL1 = CALBC1_8MHZ;
     BCSCTL3 = LFXT1S_2;             // ACLK = VLO
}

/*
 * GPIO Initialization. This block configures all the DRV8889-Q1 PINs.
 */
static inline void gpio_Init(void)
{

    // Note: All unused ports should be set to either output or input with
    //       pullup/down enabled

    P1DIR = BIT7 | STEP | BIT5 | BIT4 | BIT3 | DIR | DRVOFF | nSLEEP;       // All OUTPUT
    P1OUT = BIT7 | BIT5 | BIT4 | BIT3 | DRVOFF;                             // nSLEEP low, DIR low, STEP low, others high

    P2DIR = BIT6 | BIT5 | BIT4 | BIT3 | BIT2 | BIT1 | BIT0;                 // All OUTPUT except BIT7 for nFAULT2
    P2OUT = nFAULT_INT | BIT6 | BIT5 | BIT4 | BIT3 | BIT2 | BIT1 | BIT0;    // All high

    P3DIR = BIT7 | BIT6 | BIT4 | SCLK | SDO | SDI | nSCS;                   // All OUTPUT except bit 5 (RXD)
    P3OUT = BIT7 | BIT6 | BIT4 | SCLK | SDO | SDI | nSCS;                   // All high except bit 5 (RXD)

    P4DIR = BIT7 | BIT5 | BIT4 | BIT3 | BIT2 | BIT1 | BIT0;                 // All OUTPUT except BIT6 (nFAULT)
    P4OUT = BIT7 | nFAULT | BIT5 | BIT4 | BIT3 | BIT2 | BIT1 | BIT0;        // All high

    P5DIR = BIT7 | BIT6 | BIT5 | BIT4 | BIT3 | BIT2 | BIT1 | BIT0;          // All OUTPUT
    P5OUT = BIT7 | BIT6 | BIT5 | BIT4 | BIT3 | BIT2 | BIT1 | BIT0;          // All high

    P6DIR = VREF | BIT6 | STATUS | BIT4 | BIT3 | BIT2 | BIT1 | BIT0;        // All OUTPUT
    P6OUT = VREF | BIT6 | BIT4 | BIT3 | BIT2 | BIT1 | BIT0;                 // All high except bit 5 (status led)

    P7DIR = BIT7 | BIT6 | BIT5 | BIT4 | BIT3 | BIT2 | BIT1 | BIT0;           // All OUTPUT
    P7OUT = BIT7 | BIT6 | BIT5 | BIT4 | BIT3 | BIT2 | BIT1 | BIT0;           // All high

    P8DIR = BIT7 | BIT6 | BIT5 | BIT4 | BIT3 | BIT2 | BIT1 | BIT0;           // All OUTPUT
    P8OUT = BIT7 | BIT6 | BIT5 | BIT4 | BIT3 | BIT2 | BIT1 | BIT0;           // All high

    PADIR = BIT7 | BIT6 | BIT5 | BIT4 | BIT3 | BIT2 | BIT1 | BIT0;           // All OUTPUT
    PAOUT = BIT7 | BIT6 | BIT5 | BIT4 | BIT3 | BIT2 | BIT1 | BIT0;           // All high

    // Configure DAC
    DAC12_1CTL = DAC12SREF_3 | DAC12IR | DAC12AMP_7;                         // Select VeREF, 1x, and Amp high speed/high current

    set_DRVOFF_hi;                                                           // DRV outputs disabled

   _delay_cycles(8000);
}

/*
 * Setup Watchdog Timer as an Interval Timer(250ms)
 * Timer Interval to Blink the STATUS LED(P2.7)
 */
static inline void watchdogTimer_Init(void)
{
//  WDTCTL = WDT_ADLY_250;             /** < Configure time interval*/
  WDTCTL = WDT_MDLY_32;               // Set Watchdog Timer interval to 4ms
  IE1 |= WDTIE;                       // Watchdog Interrupt enable
}

/*******************************************************************************
* API IMPLEMENTATION
*******************************************************************************/
void drv8889q1_init()   // Initialize GUI variables
{

    gDRV8889_Obj.Firmware_Version = 1;
    gDRV8889_Obj.nFault = 1;
    gDRV8889_Obj.nSleep = 0;
    gDRV8889_Obj.gDRVOFF = 1;
    gDRV8889_Obj.DeviceID = 8899;
    gDRV8889_Obj.ClrFlt = 0;
}

void UpdateGUI_Register()   // Routine for updating registers from the GUI
{
    uint16_t drvDataNew;
    uint8_t drvRegAddr;
    if(gDRV8889_Obj.nSleep == 1)
    {
        /*Device ID*/

        /* Write Register Values*/
        if(gDRV8889_Obj.ClrFlt && (gDRV8889_Obj.nFault == 0))
        {
            gDeviceSpiReg.ctrl4_reg |= CLR_FLT_MASK;
            SPI_writeRegister(SPI_REG_CTRL4, (uint16_t)  gDeviceSpiReg.ctrl4_reg);
            gDRV8889_Obj.nFault = 1;
            gDRV8889_Obj.ClrFlt = 0;
        }

        if(gDRV8889_Obj.ManWriteCmd)
          {
            // Custom Write
            drvRegAddr = gDRV8889_Obj.ManWriteAddr;
            drvDataNew = gDRV8889_Obj.ManWriteData;

            SPI_writeRegister((uint8_t)drvRegAddr, (uint16_t) drvDataNew);
            switch(drvRegAddr)
            {
                case SPI_REG_FAULT_STATUS:          /* Fault Status Register */
                gDeviceSpiReg.fault_status_reg = drvDataNew;       /* Fault Status Register, Address = 0x00h*/
                break;

                case SPI_REG_DIAG_STATUS_1:        /* Diagnostic register 1 for OCP */
                gDeviceSpiReg.diag_status1_reg = drvDataNew;     /* Diagnostic register 1 for OCP, Address = 0x01h*/
                break;

                case SPI_REG_DIAG_STATUS_2:        /* Diagnostic register 2 for other faults and warnings */
                gDeviceSpiReg.diag_status2_reg = drvDataNew;     /* Diagnostic register 2 for other faults and warnings, Address = 0x02h*/
                break;

                case SPI_REG_CTRL1:                 /* CTRL1 register, slew rate */
                gDeviceSpiReg.ctrl1_reg = drvDataNew;     /* CTRL1 register, Address = 0x03h*/
                break;

                case SPI_REG_CTRL2:                 /* CTRL2 register - Disable, TOFF, DECAY */
                gDeviceSpiReg.ctrl2_reg = drvDataNew;     /* CTRL2 register, Address = 0x04h*/
                break;

                case SPI_REG_CTRL3:                 /* CTRL3 register - Step, dir, microstep */
                gDeviceSpiReg.ctrl3_reg = drvDataNew;     /* CTRL3 register, Address = 0x05h*/
                break;

                case SPI_REG_CTRL4:                 /* CTRL4 register - Clear fault, lock, OL, OCP, OTSD, OTW */
                gDeviceSpiReg.ctrl4_reg = drvDataNew;     /* CTRL4 register, Address = 0x06h*/
                break;

                case SPI_REG_CTRL5:                 /* CTRL5 register - Stall detection configuration */
                gDeviceSpiReg.ctrl5_reg = drvDataNew;      /* CTRL5 register, Address = 0x07h*/
                break;

                case SPI_REG_CTRL6:                 /* CTRL6 register - Stall threshold */
                gDeviceSpiReg.ctrl6_reg = drvDataNew;        /* CTRL6 register, Address = 0x08h*/
                break;

                case SPI_REG_CTRL7:                 /* CTRL7 register - Torque count */
                gDeviceSpiReg.ctrl7_reg = drvDataNew;        /* CTRL7 register, Address = 0x09h*/
                break;

                case SPI_REG_CTRL8:         /* CTRL8 register - UTW, Rev ID */
                gDeviceSpiReg.ctrl8_reg = drvDataNew;        /* CTRL8 register, Address = 0x0Ah*/
                break;
                default: break;

            }// End of Switch
            gDRV8889_Obj.ManWriteCmd = false;
          } // End of Write Cmd

        if(gDRV8889_Obj.ReadCmd){

            gDeviceSpiReg.fault_status_reg = SPI_readRegister((uint8_t)SPI_REG_FAULT_STATUS);

            gDeviceSpiReg.diag_status1_reg = SPI_readRegister((uint8_t)SPI_REG_DIAG_STATUS_1);

            gDeviceSpiReg.diag_status2_reg = SPI_readRegister((uint8_t)SPI_REG_DIAG_STATUS_2);

            gDeviceSpiReg.ctrl1_reg = SPI_readRegister((uint8_t)SPI_REG_CTRL1);

            gDeviceSpiReg.ctrl2_reg = SPI_readRegister((uint8_t)SPI_REG_CTRL2);

            gDeviceSpiReg.ctrl3_reg = SPI_readRegister((uint8_t)SPI_REG_CTRL3);

            gDeviceSpiReg.ctrl4_reg = SPI_readRegister((uint8_t)SPI_REG_CTRL4);

            gDeviceSpiReg.ctrl5_reg = SPI_readRegister((uint8_t)SPI_REG_CTRL5);

            gDeviceSpiReg.ctrl6_reg = SPI_readRegister((uint8_t)SPI_REG_CTRL6);

            gDeviceSpiReg.ctrl7_reg = SPI_readRegister((uint8_t)SPI_REG_CTRL7);

            gDeviceSpiReg.ctrl8_reg = SPI_readRegister((uint8_t)SPI_REG_CTRL8);

            gDRV8889_Obj.ReadCmd = false;

        }
        if(gDRV8889_Obj.ManReadCmd)
        {

           // Custom Read
            drvRegAddr = gDRV8889_Obj.ManReadAddr;
            drvDataNew = SPI_readRegister((uint8_t)drvRegAddr);

            gDRV8889_Obj.ManReadCmd = false;
            gDRV8889_Obj.ManReadData = drvDataNew;
            switch(drvRegAddr)
            {
                case SPI_REG_FAULT_STATUS:          /* Fault Status Register */
                gDeviceSpiReg.fault_status_reg = drvDataNew;       /* Fault Status Register, Address = 0x00h*/
                break;

                case SPI_REG_DIAG_STATUS_1:        /* Diagnostic register 1 for OCP */
                gDeviceSpiReg.diag_status1_reg = drvDataNew;     /* Diagnostic register 1 for OCP, Address = 0x01h*/
                break;

                case SPI_REG_DIAG_STATUS_2:        /* Diagnostic register 2 for other faults and warnings */
                gDeviceSpiReg.diag_status2_reg = drvDataNew;     /* Diagnostic register 2 for other faults and warnings, Address = 0x02h*/
                break;

                case SPI_REG_CTRL1:                 /* CTRL1 register, slew rate */
                gDeviceSpiReg.ctrl1_reg = drvDataNew;     /* CTRL1 register, Address = 0x03h*/
                break;

                case SPI_REG_CTRL2:                 /* CTRL2 register - Disable, TOFF, DECAY */
                gDeviceSpiReg.ctrl2_reg = drvDataNew;     /* CTRL2 register, Address = 0x04h*/
                break;

                case SPI_REG_CTRL3:                 /* CTRL3 register - Step, dir, microstep */
                gDeviceSpiReg.ctrl3_reg = drvDataNew;     /* CTRL3 register, Address = 0x05h*/
                break;

                case SPI_REG_CTRL4:                 /* CTRL4 register - Clear fault, lock, OL, OCP, OTSD, OTW */
                gDeviceSpiReg.ctrl4_reg = drvDataNew;     /* CTRL4 register, Address = 0x06h*/
                break;

                case SPI_REG_CTRL5:                 /* CTRL5 register - Stall detection configuration */
                gDeviceSpiReg.ctrl5_reg = drvDataNew;      /* CTRL5 register, Address = 0x07h*/
                break;

                case SPI_REG_CTRL6:                 /* CTRL6 register - Stall threshold */
                gDeviceSpiReg.ctrl6_reg = drvDataNew;        /* CTRL6 register, Address = 0x08h*/
                break;

                case SPI_REG_CTRL7:                 /* CTRL7 register - Torque count */
                gDeviceSpiReg.ctrl7_reg = drvDataNew;        /* CTRL7 register, Address = 0x09h*/
                break;

                case SPI_REG_CTRL8:         /* CTRL8 register - UTW, Rev ID */
                gDeviceSpiReg.ctrl8_reg = drvDataNew;        /* CTRL8 register, Address = 0x0Ah*/
                break;
                default: break;
            } // End of Switch
        }// End of Read Cmd
    }/* nSleep*/
}

void mcu_init()
{
  // Stop Watchdog Timer
  WDTCTL = WDTPW | WDTHOLD;

  // Setup CLKs
  clock_Init();
  uartInit();
  gpio_Init();
  SPI_Init();
  watchdogTimer_Init();
}
